@@ -5,6 +5,7 @@ pd.set_option("display.max_rows", 500)
 warnings.filterwarnings('ignore')
 slicer = pd.IndexSlice
 from lifelines.fitters.kaplan_meier_fitter import KaplanMeierFitter
+from typing import Optional, List, Union
 
 
 def event_specific_prediction_error(pred_df: pd.DataFrame,
@@ -74,6 +75,35 @@ def prediction_error(pred_df: pd.DataFrame,
     return prediction_error.T
 
 
+def events_auc_at_t(pred_df: pd.DataFrame,
+                    event_type_col: str = 'J',
+                    duration_col: str ='X') -> pd.DataFrame:
+    """
+    This function implements the calculation of the events AUC at t.
+
+    Args:
+        pred_df (pd.DataFrame): Data to calculate prediction error.
+                                Must contain the observed duration and event-type, and the cumulative incident function prediction results.
+                                See TwoStagesFitter.predict_cumulative_incident_function()
+        duration_col (str): Last follow up time column name (must be a column in pred_df).
+        event_type_col (str): The event type column name (must be a column in df),
+                              Right-censored sample (i) is indicated by event value 0, df.loc[i, event_type_col] = 0.
+    Returns:
+        event_auc_at_t_df (pd.DataFrame): events auc at t results.
+    """
+
+    events = [e for e in pred_df[event_type_col].unique() if e != 0]
+    event_auc_at_t_df = pd.DataFrame()
+    for event in sorted(events):
+        event_auc_at_t_df = pd.concat([event_auc_at_t_df,
+                                       event_specific_auc_at_t_all(pred_df=pred_df,
+                                                                   event=event,
+                                                                   event_type_col=event_type_col,
+                                                                   duration_col=duration_col)],
+                                       axis=1)
+    return event_auc_at_t_df.T
+
+
 def event_specific_auc_at_t(pred_df: pd.DataFrame,
                             event: int,
                             t: int,
@@ -100,6 +130,8 @@ def event_specific_auc_at_t(pred_df: pd.DataFrame,
     no_event_at_t_df = no_event_at_t_df[~((no_event_at_t_df[event_type_col] == event) &
                                           (no_event_at_t_df[duration_col] == t))]
     total_t = (len(event_observed_at_t_df)*len(no_event_at_t_df))
+    if total_t == 0:
+        return np.nan
     correct_order = 0
     for i_idx, i_row in event_observed_at_t_df.iterrows():
         pi_ij = i_row.loc[f'prob_j{event}_at_t{t}']
@@ -131,7 +163,7 @@ def event_specific_auc_at_t_all(pred_df: pd.DataFrame,
     for t in sorted(pred_df[duration_col].unique())[:-1]:
         res[t] = event_specific_auc_at_t(pred_df=pred_df, event=event, t=t,
                                          event_type_col=event_type_col, duration_col=duration_col)
-    return pd.Series(res)
+    return pd.Series(res, name=event)
 
 
 def event_specific_auc_weights(pred_df: pd.DataFrame,
@@ -168,7 +200,8 @@ def event_specific_auc_weights(pred_df: pd.DataFrame,
 def event_specific_integrated_auc(pred_df: pd.DataFrame,
                                   event: int,
                                   event_type_col: str = 'J',
-                                  duration_col: str = 'X') -> float:
+                                  duration_col: str = 'X',
+                                  weights: Union[pd.Series, None] = None) -> float:
     """
     This function implements the calculation of the event specific integrated auc.
 
@@ -180,6 +213,7 @@ def event_specific_integrated_auc(pred_df: pd.DataFrame,
         duration_col (str): Last follow up time column name (must be a column in pred_df).
         event_type_col (str): The event type column name (must be a column in df),
                               Right-censored sample (i) is indicated by event value 0, df.loc[i, event_type_col] = 0.
+        weights (pd.Series): Optional. Weights vector with time as index and weight as value. Length must be the number of possible event times.
     Returns:
         result (float): integrated AUC results.
     """
@@ -187,10 +221,11 @@ def event_specific_integrated_auc(pred_df: pd.DataFrame,
     auc_at_t = event_specific_auc_at_t_all(pred_df=pred_df, event=event,
                                            event_type_col=event_type_col,
                                            duration_col=duration_col)
-    weights = event_specific_auc_weights(pred_df=pred_df, event=event,
-                                         event_type_col=event_type_col,
-                                         duration_col=duration_col)
-    result = auc_at_t.dot(weights)
+    if weights is None:
+        weights = event_specific_auc_weights(pred_df=pred_df, event=event,
+                                             event_type_col=event_type_col,
+                                             duration_col=duration_col)
+    result = auc_at_t.dot(weights.sort_index())
     return result
 
 
@@ -219,3 +254,29 @@ def global_auc(pred_df: pd.DataFrame,
             pred_df=pred_df, event=event, event_type_col=event_type_col,
             duration_col=duration_col)
     return global_auc
+
+
+def events_integrated_auc(pred_df: pd.DataFrame, event_type_col: str = 'J',
+                          duration_col: str ='X') -> dict:
+    """
+    This function implements the calculation of the integrated AUC to all events.
+
+    Args:
+        pred_df (pd.DataFrame): Data to calculate prediction error.
+                                Must contain the observed duration and event-type, and the cumulative incident function prediction results.
+                                See TwoStagesFitter.predict_cumulative_incident_function()
+        duration_col (str): Last follow up time column name (must be a column in pred_df).
+        event_type_col (str): The event type column name (must be a column in df),
+                              Right-censored sample (i) is indicated by event value 0, df.loc[i, event_type_col] = 0.
+    Returns:
+        integrated_auc (dict): integrated AUC results.
+    """
+
+    events = [e for e in pred_df[event_type_col].unique() if e != 0]
+    integrated_auc = {}
+    for event in sorted(events):
+        integrated_auc[event] = event_specific_integrated_auc(pred_df=pred_df,
+                                                              event=event,
+                                                              event_type_col=event_type_col,
+                                                              duration_col=duration_col)
+    return integrated_auc
